from matchescu.extraction._base import EntityReferenceExtraction
from matchescu.extraction._record import RecordExtraction
from matchescu.extraction._samplers import single_record
from matchescu.extraction._traits import Traits

__all__ = [
    "EntityReferenceExtraction",
    "RecordExtraction",
    "Traits",
    "single_record",
]



import itertools
from collections.abc import Iterable

from matchescu.data import Record
from matchescu.typing import EntityReferenceIdentifier


class EntityReference(Record):
    id: EntityReferenceIdentifier

    def __init__(self, identifier: EntityReferenceIdentifier, value: Iterable):
        super().__init__(value)
        self.id = identifier

    def __eq__(self, __value):
        if not isinstance(__value, EntityReference):
            return False
        return __value.id == self.id

    def __ne__(self, __value):
        return not self.__eq__(__value)

    def __repr__(self):
        return "EntityReference(id={})".format(repr(self.id))

    def __hash__(self):
        return hash(self.id)

    def __dir__(self):
        return list(itertools.chain(super().__dir__(), self._attr_names.keys()))

    def as_dict(self) -> dict:
        return {
            attr_name: self._attr_values[attr_idx]
            for attr_name, attr_idx in self._attr_names.items()
        }



from typing import Iterable
from matchescu.extraction._base import EntityReferenceExtraction

from matchescu.data import Record


class RecordExtraction(EntityReferenceExtraction[Record]):
    def _merge_records(self, records: Iterable[Record]) -> Record:
        return Record.merge(records)



from collections.abc import Sequence, Iterator
from os import PathLike
from pathlib import Path

import polars as pl

from matchescu.typing import Trait

from matchescu.data import Record


class CsvDataSource:
    def __init__(
        self,
        file_path: str | PathLike,
        traits: Sequence[Trait],
        has_header: bool = True,
    ):
        file_path = Path(file_path)
        self.name = file_path.name.replace(file_path.suffix, "")
        self.traits = traits

        self.__file_path = file_path
        self.__df: pl.DataFrame | None = None
        self.__header = has_header

    def read(self) -> "CsvDataSource":
        self.__df = pl.read_csv(
            self.__file_path, ignore_errors=True, has_header=self.__header
        )
        return self

    def __iter__(self) -> Iterator[Record]:
        if self.__df is None:
            return iter([])
        return iter(map(Record, self.__df.iter_rows(named=True)))

    def __len__(self) -> int:
        return self.__df.shape[0] if self.__df is not None else 0



from typing import Generator

from matchescu.typing import DataSource


def single_record(data_source: DataSource) -> Generator[list, None, None]:
    """Sample records one at a time from the data source."""
    yield from map(lambda x: [x], data_source)
